        list p=16f628a
    
; PILio12.asm
; PIL-IO version 1.2 - mini HPIL/GPIO + HPIL/serial interface
; History:
; 29/05/10 creation based on PIL-Box 1
;          dual device implementation
; 21/06/10 debug
; 09/04/12 version 1.1:
;  no more brownout (can run on 3xAA batteries)
;  LPD implementation (0.2mA in power down instead of 3mA in idle mode)
;  GET implementation (I/O3 line is TRGO and is pulsed down for 800us)
;  SSRQ, FREN for future usage
; 04/04/14 - 15/05/14 version 1.2:
;  basic serial control line management:
;    GTL to PILSER/PILPIO enables/disables serial ctrl lines
;    I/O0 is CTSI, I/O1 is RTSO and I/O2 is SRQI 
;    SRQ bit in HPIL frames is set if SRQI is low and serial ctrl lines are enabled
;    serial status now includes CTS (bit 2) and SRQ (bit 6)
;    CTS is used for output handshake: transmit is delayed until CTS is true (CTSI low),
;      but if a new HPIL frame arrives, then the waiting loop is aborted.
;    RTS is used for input handshake: RTS is false (RTSO high) when 
;      the 64-byte input buffer is almost full (more than 56 bytes in it)
;    waiting time for a byte on serial
;    16-byte output buffer
;    status bits RXA, OVR, ERR cleared when buffer empty
;  improve frame decoding (from PIL-Box 1.43)
; To do: 
;  dynamic 16/64 byte buffer fence 
;  customizable AID 
;  customizable serial configuration (RTSO, CTSI, SRQI, baud rate, status)
;  I2C and SPI support ":Ran", ":Wanxxx", answers "Rnddd", "R0", "Wn", "W0"
;  integration into the PIL-Box mailbox mode

        #include <P16F628A.INC>
;
; config bits: HS osc, watchdog on, powerup timer on, code protect on,
; brownout disabled, master clear disabled, low voltage prog disabled, data ee protect disabled.

        __config 0x0106   ; 16F628A


;
; 20-6f: bank of 80 bytes
BUF     equ     0x20        ; buffer for IL frame samples (20-5f)
DECFLG  equ     0x60        ; decoding flags
CPTBIT  equ     0x62        ; bit counter
CPTSAM  equ     0x63        ; sample counter
FRREGH  equ     0x64        ; input/output frame shift registers
FRREGL  equ     0x65
SAVEW   equ     0x6f        ; save W during IT

; 70-7f: commun area 
ILFLG   equ     0x70            ; IL flags 
FRREC   equ     0     ; frame received
FRAV    equ     1     ; frame available  
FLPD    equ     2     ; LPD received
SSRQ    equ     3     ; set SRQ bit in IDY
FREN    equ     4     ; remote mode
FPBYT   equ     5     ; pending byte (in SBYTE)
;
CPTB    equ     0x71            ; general purpose counter
FRAMEH  equ     0x72            ; IL 11-bit frame in 72-73
FRAMEL  equ     0x73
SBYTE   equ     0x74           ; byte from/to serial
; variables for devices:
ILADDR  equ     0x75     ; IL device base address (bits 0-4). bit 7 means addresses assigned
ILSTAT  equ     0x76     ; IL device state bits
; FLST    FTLK
;    0       0     idle
;    1       0     listener
;    0       1     adressed talker
;    1       1     active talker
FLST    equ     7     ; addressed 
FTLK    equ     6     ; talker
FSDA    equ     5     ; addressed to send data
FSDI    equ     4     ; addressed to send data with PTSDI
FSER    equ     0     ; 0: PIO, 1:serial I/O is addressed
;
SERSP   equ     0x77     ; UART IL serial poll byte
SERRX   equ     0    ; Rx byte available
SERRXA  equ     1    ; Rx LF received
SERCTS  equ     2    ; CTS status
SEROVR  equ     3    ; Rx buffer overrun
SERERR  equ     4    ; Rx frame error
SERSRQ  equ     6    ; SRQ
;
PTSDI   equ     0x78    ; output pointer for device ID
PTSER   equ     0x79    ; pointer for serial data
NBSER   equ     0x7a    ; number of bytes in serial buffer
SERFLG  equ     0x7b    ; serial flags
FCTS    equ     0     ; enable CTSI serial control line
FRTS    equ     1     ; enable RTSO serial control line
FSRQ    equ     2     ; enable SRQ if SRQI is low
FDOUT   equ     3     ; data ready to be send
PTSERO  equ     0x7c
NBSERO  equ     0x7d
;
ACCY    equ     0x7e    ; temporary accumulator
ACCX    equ     0x7f    ; temporary accumulator

; bank 2 allocation
BUFOUT  equ     0xa0   ; 16-byte serial output buffer (a0-af) - not used
BUFSER  equ     0xb0   ; 64-byte serial input  buffer (b0-ef)
; 



;FIX1A   equ     1      ; enable HP71B HPIL 1A fix
;HSPEED  equ     1      ; high serial speed: 115k

TRIGO   equ     7      ; Trigger output          - RB7 - shared with I/O 3
SRQI    equ     6      ; Service Request input   - RB6 - shared with I/O 2
RTSO    equ     5      ; RTS output control line - RB5 - shared with I/O 1
CTSI    equ     4      ; CTS input  control line - RB4 - shared with I/O 0
LEDOFF  equ     4      ; LED command - RA4
MLED    equ     0x10   ; bit mask for LED

DEFADDR equ  8         ; default base address after AAU
AIDPIO  equ  0x40      ; GPIO AID
AIDSER  equ  0x42      ; serial AID


;
        org     0
        goto    Start

        org     4
Intr
; interrupt routine to sample the HPIL frame
        movwf   SAVEW       ; save W
; fast entry point for fix
Intr1        
     variable i
i=0
     while i<D'64'   ; 128 samples stored in 64 registers
        movf    CMCON,W     ; ABxx11xx
        movwf   BUF+i
        swapf   CMCON,w     ; 11xxCDxx
        andwf   BUF+i,f     ; ABxxCDxx
i+=1
     endw
        bcf     PIR1, CMIF   ; reset IT cause
        bsf     ILFLG,FRREC
        movf    SAVEW,W      ; restore W and set STATUS,Z accordingly
        retfie


GetSDI
        andlw   0x1f
        addwf   PCL,f
        dt      "PILPIO1C",0x0d,0x0a,0   ; 0;11
        dt      "PILSER1C",0x0d,0x0a,0   ; 11;11
        dt      "PIL-IO 1.2",0x0d,0x0a   ; 22;12
        dt      "(c) J-F GARNIER 2014",0x0d,0x0a,0  ; 22
; 1234567890123456789012
; PIL-IO 1.0 JFG (c)2010      

Start
        clrwdt
        movlw   B'00000000'     ; port outputs
        movwf   PORTA
        movwf   PORTB
        bsf     STATUS, RP0     ; *****************
        movlw   B'11101111'     ;make port a4 output
        movwf   TRISA           ;       /
        movlw   B'11110010'     ;make port b0,2,3 outputs
        movwf   TRISB           ;       /
        bcf     OPTION_REG,7    ; weak pull-up on port B
        call    Dly100us        ; delay 100us

; -- setup of the UART
; BRG programmed at high speed to 9.6k/19.2k/57.6k/115.2k  
;                 SPBRG code=     129 / 64  / 21  / 10    
        bcf     STATUS, RP0  ; *************************
        movlw   D'129'  
  ifdef HSPEED        
        movlw   D'10'
  endif        
        bsf     STATUS, RP0  ; *************************
        movwf   SPBRG
; setup Rx and Tx
        movlw   0x24         ; Tx enable, async high speed
        movwf   TXSTA        
        bcf     STATUS, RP0  ; *************************
        movlw   0x90         ; Serial enable, continous receive
        movwf   RCSTA        

; -- setup of the comparators
        movlw   0x1c         ;  mode 100, C1INV
        movwf   CMCON
        nop
        movf    CMCON,W       ; read

; enable INT on comparator change
        bsf     STATUS, RP0  ; *************************
        movlw   0x40            ; CMIE
        movwf   PIE1
        bcf     STATUS, RP0  ; *************************
        movlw   0x40            ; PEIE
        movwf   INTCON

; init variables        
        clrf    ILFLG
        clrf    ILSTAT
        clrf    SERFLG
        movlw   DEFADDR
        movwf   ILADDR
        call    clrser

; make blink the LED
        call    blinkLED

        bsf     INTCON, GIE ; enable intr

        
; ======== main loop ===========
Main
; internal loop, waiting for IL frame, or serial byte
        clrwdt
        btfsc   PIR1,RCIF
        call    inserial
        btfsc   ILFLG,FRREC
        call    insamp
        btfsc   ILFLG,FRAV
        call    inIL
        btfsc   SERFLG,FDOUT
        call    sndserial
        ; update SSRQ
        bcf     ILFLG,SSRQ
        btfss   SERFLG,FSRQ  ; only if enabled
        goto    Main
        btfss   PORTB,SRQI
        bsf     ILFLG,SSRQ
        goto    Main
; ================================


clrpio
        bsf     STATUS, RP0     ; *****************
        movlw   B'11110010'     ;make port b0,2,3 outputs
        movwf   TRISB           ;       /
        bcf     STATUS, RP0     ; *************************
        return

clrser
        clrf    NBSER
        clrf    PTSER
        clrf    NBSERO
        clrf    PTSERO
        clrf    SERSP
        bcf     SERFLG,FDOUT
        btfss   SERFLG,FRTS
        goto    clrser9
        bsf     STATUS, RP0     ; *****************
        bcf     TRISB,RTSO      ; RTS true (low)
        bcf     STATUS, RP0     ; *****************
clrser9 return


powerdn
        bcf     INTCON, GIE   ; disable intr
        sleep
        nop
        nop
        btfss   PIR1,CMIF
        goto    powerdn
        bcf     ILFLG,FLPD      
        bsf     INTCON, GIE   ; enable intr
        return      

      
trigger
; pulse IO3 low 800us
        bsf     STATUS, RP0     ; *****************
        btfss   TRISB,7
        goto    trig9
        bcf     TRISB,7        
        call    Dly800us
        bsf     TRISB,7
trig9   bcf     STATUS, RP0     ; *****************
        return      
      
      
; -------------------------------        
; IL frame samples received.
; -------------------------------
insamp
        bcf     INTCON, GIE   ; disable intr
        bcf     PORTA,LEDOFF       ; LED on
        bcf     ILFLG,FRREC
        call    decframe      ; decode frame into FRAMEH/L,  FRAV is set if ok
        bsf     INTCON, GIE   ; enable intr
        bsf     PORTA,LEDOFF   ; LED off
        return
        
; -------------------------------        
; IL frame available.
; -------------------------------
inIL    bcf     INTCON, GIE   ; disable intr
        bcf     PORTA,LEDOFF       ; LED on
        bcf     ILFLG,FRAV
        
        ; according to frame type:
        btfss   FRAMEH,2   ; control frame?
        goto    TRDOE      ;   no (DOE), processes it
        btfsc   FRAMEH,1   ; CMD or RDY?
        goto    TRIDY      ;   no (IDY), re-transmit
        btfss   FRAMEH,0   ; RDY?
        goto    TRCMD      ;   no (CMD), processes it
        movf    FRAMEL,W   ; 
        btfss   STATUS,Z   ; RFC?
        goto    TRRDY      ;   no (RDY), process it
        ;   yes (RFC), re-transmit
        call    sendIL     ; send 
        bsf     PORTA,LEDOFF   ; LED off        
        btfsc   ILFLG,FLPD
        call    powerdn        
        return
        
TRIDY   btfsc   ILFLG,SSRQ
        bsf     FRAMEH,0   ; set bit SRQ in IDY frame
        goto    TR11
                
TR10    btfsc   FRAMEH,3
        goto    TR11
        btfsc   ILFLG,SSRQ
        bsf     FRAMEH,0   ; set bit SRQ in DOE frame
TR11    call    sendIL     ; send HPIL frame (intr are enabled at return)
        bsf     PORTA,LEDOFF   ; LED off        
        return

TR12    ; *** exit point for serial DOE (pending byte):
        bsf     PORTA,LEDOFF   ; LED off     
        bsf     INTCON, GIE   ; enable intr   
        return

        ; ------------------------------------------        
        ; frames to be processed by the application
        ; ------------------------------------------        

; -------------------------------        
; data or end frames
; -------------------------------
TRDOE
        bcf     ILFLG,FLPD
        bcf     ILFLG,FPBYT
        btfss   ILSTAT,FLST   ; addressed?
        goto    TRDOE9        ; no, return
        btfss   ILSTAT,FTLK   ; yes. active talker?
        goto    TRDOE2        ; no, listener
        ; active talker
        btfss   ILSTAT,FSDA   ; SDA or SDI on going?
        goto    TRDOE7        ; no, send EOT
        ; yes (SDA or SDI)
        btfss   ILSTAT,FSDI   ; SDI?
        goto    TRDOE1        ; no. manage SDA
        ; SDI
        ; send data from ROM using PTSDI
        movlw   HIGH GetSDI
        movwf   PCLATH
        movf    PTSDI,W
        btfsc   STATUS,Z ; PTSDI=0?
        goto    TRDOE7   ; yes, send EOT
        call    GetSDI
        incf    PTSDI
        iorlw   0
        btfss   STATUS,Z ; last byte (=0)?
        goto    TRDOE8   ; no send it
        clrf    PTSDI
        goto    TRDOE7   ; else send EOT
        
TRDOE1  ; send data from RAM using PTSER
        call    getserb
        btfsc   STATUS,Z
        goto    TRDOE7    ; no data, EOT
        goto    TRDOE8
        
TRDOE2  ; listener
        btfsc   ILSTAT,FSER
        goto    TRDOE3
        ; PIO
        movf    FRAMEL,W
        sublw   0x2f  ; 2f-w; Cy set if >=0 ie w<=2f 
        btfsc   STATUS,C
        goto    TRDOE9
        movf    FRAMEL,W
        sublw   0x40  ; 40-w; Cy set if >=0 ie w<=40 
        swapf   FRAMEL,W
        btfss   STATUS,C
        addlw   0x90   ; convert 'A'-'F' to 0xa-0xf
        xorlw   0xf0   ; invert cause a bit 1 means output (low)
        andlw   0xf0
        iorlw   2      ; keep RB0-RB3 unchanged
        bsf     STATUS, RP0     ; *****************
        movwf   TRISB
        bcf     STATUS, RP0     ; *****************
        goto    TRDOE9
TRDOE3  ; serial
        btfss   ILFLG,FREN
        goto    TRDOE4
        ; BR (baud rate) CL (ctrl line mode) DI (dev ID) AI (acc ID) */
        goto    TRDOE9
TRDOE4  movf    FRAMEL,W
        call    putserial
        btfsc   ILFLG,FPBYT
        goto    TR12        ; exit point for serial pending byte
        goto    TR10


TRDOE7  bcf     ILSTAT,FLST  ; remains addressed talker
        bcf     ILSTAT,FSDA
        bcf     ILSTAT,FSDI
        movlw   5
        movwf   FRAMEH
        movlw   0x40    ; EOT
TRDOE8  movwf   FRAMEL
TRDOE9  goto    TR10   ; return
        
        
        
; -------------------------------        
; ready frames
; -------------------------------
TRRDY
        bcf     ILFLG,FLPD
        bcf     ILFLG,FPBYT
        btfsc   FRAMEL,7
        goto    TRAAD
        ; 
        btfss   ILSTAT,FTLK
        goto    TRSOT9
TRSOT   movf    FRAMEL,W
        sublw   0x42        ; 66  NRD ?
        btfss   STATUS,Z
        goto    TRSOT1
        bcf     ILSTAT,FSDA    ; abort transfer
        bcf     ILSTAT,FSDI
        goto    TRSOT9
TRSOT1  movf    FRAMEL,W
        sublw   0x60         ; 96  SDA
        btfss   STATUS,Z
        goto    TRSOT2
        btfss   ILSTAT,FSER
        goto    TRSOT7        ; PIO: send EOT
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        bsf     ILSTAT,FSDA
        bcf     ILSTAT,FSDI
        ; get byte from serial buffer
        call    getserb
        btfsc   STATUS,Z
        goto    TRSOT7    ; no data, EOT
        goto    TRSOT8
TRSOT2  movf    FRAMEL,W
        sublw   0x61           ; 97   SST
        btfss   STATUS,Z
        goto    TRSOT3
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        ; read PIO stat
        swapf   PORTB,W
        andlw   0x0f
        xorlw   0x0f
        btfss   ILSTAT,FSER
        goto    TRSOT8
        ; or get serial status byte
        bcf     SERSP,SERCTS
        btfss   PORTB,CTSI    ; update CTS
        bsf     SERSP,SERCTS
        bcf     SERSP,SERSRQ
        btfss   PORTB,SRQI    ; update SRQ
        bsf     SERSP,SERSRQ
        ; get status and update self-clearing bits
        movf    SERSP,W     
        bcf     SERSP,SEROVR
        bcf     SERSP,SERERR
        bcf     SERSP,SERRXA
        goto    TRSOT8
TRSOT3  movf    FRAMEL,W
        sublw   0x62             ; 98   SDI
        btfss   STATUS,Z
        goto    TRSOT4
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        bsf     ILSTAT,FSDA
        bsf     ILSTAT,FSDI
        movlw   HIGH GetSDI
        movwf   PCLATH
        clrw
        btfsc   ILSTAT,FSER
        movlw   D'11'
        movwf   PTSDI
        call    GetSDI
        incf    PTSDI
        goto    TRSOT8
TRSOT4  movf    FRAMEL,W
        sublw   0x63                  ; 99   SAI
        btfss   STATUS,Z
        goto    TRSOT9
        bsf     ILSTAT,FLST   ; FTLK+LST=active talker
        movlw   AIDPIO
        btfsc   ILSTAT,FSER
        movlw   AIDSER
        goto    TRSOT8

TRSOT7  bcf     ILSTAT,FLST  ; remains addressed talker
        bcf     ILSTAT,FSDA
        movlw   5
        movwf   FRAMEH
        movlw   0x40    ; EOT
        movwf   FRAMEL
        goto    TRSOT9
TRSOT8  movwf   FRAMEL
        clrf    FRAMEH
TRSOT9  goto    TR10      ; return

TRAAD   ; AAD frames
        movf    FRAMEL,w
        sublw   0x9C  ; 9C-addr; Cy set if >=0 ie addr<=9C
        btfss   STATUS,C
        goto    TRAAD9
TRAAD1  btfsc   ILADDR,7  ; already an assigned address?
        goto    TRAAD9    ; yes
        movf    FRAMEL,w  ; no, take it as base addr
        movwf   ILADDR
        incf    FRAMEL
        incf    FRAMEL    ; takes 2 addresses
TRAAD9  goto    TR11      ; return

        
        

; -------------------------------        
; command frames
; -------------------------------
TRCMD   ; CMD frames
        bcf     ILFLG,FLPD
        bcf     ILFLG,FPBYT
        movf    FRAMEL,W
        andlw   0xe0         ; keep bits 7-5
        btfss   STATUS,Z
        goto    TRCMD1
TRCMD0  ; cmd group 0
        movf    FRAMEL,W
        sublw   1
        btfss   STATUS,Z
        goto    TRCMD01
        ; GTL
        btfss   ILSTAT,FLST
        goto    TRCMD09
        bcf     ILFLG,FREN
        bcf     SERFLG,FSRQ
        bcf     SERFLG,FCTS
        bcf     SERFLG,FRTS
        btfss   ILSTAT,FSER
        goto    TRCMD09
        bsf     SERFLG,FSRQ
        bsf     SERFLG,FCTS
        bsf     SERFLG,FRTS
        call    hndRTS        ; manage RTS hardware handshake line 
        goto    TRCMD09
TRCMD01 movf    FRAMEL,W
        sublw   4
        btfss   STATUS,Z
        goto    TRCMD02
        ; SDC
        btfss   ILSTAT,FLST
        goto    TRCMD09
        btfss   ILSTAT,FSER
        call    clrpio
        btfsc   ILSTAT,FSER
        call    clrser
        goto    TRCMD09
TRCMD02 movf    FRAMEL,W
        sublw   8
        btfss   STATUS,Z
        goto    TRCMD03
        ; GET
        btfss   ILSTAT,FLST
        goto    TRCMD09
;;        btfss   ILSTAT,FSER   removed v1.2
        call    trigger
        goto    TRCMD09
TRCMD03 movf    FRAMEL,W
        sublw   0x14
        btfss   STATUS,Z
        goto    TRCMD09
        ; DCL
        call    clrpio
        call    clrser       
TRCMD09 goto    TR11
;        
TRCMD1  addlw   0xe0
        btfss   STATUS,Z
        goto    TRCMD2
        ; LAD        
        movf    FRAMEL,W
        addlw   1
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD11
        ; UNL
        btfss   ILSTAT,FTLK
        clrf    ILSTAT
        goto    TRCMD19
TRCMD11 movf    ILADDR,W
        subwf   FRAMEL,W   ; FRAMEL-ILADDR
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD12
        ; MLA (PIO)
        clrf    ILSTAT
        bsf     ILSTAT,FLST
        goto    TRCMD19
TRCMD12 addlw   -1
        btfss   STATUS,Z
        goto    TRCMD19
        ; MLA (SER)
        clrf    ILSTAT
        bsf     ILSTAT,FLST
        bsf     ILSTAT,FSER
TRCMD19 goto    TR11

TRCMD2  addlw   0xe0
        btfss   STATUS,Z
        goto    TRCMD4
        ; TAD
        ; if talker, reset talker states 
        btfsc   ILSTAT,FTLK
        clrf    ILSTAT
        ; 
        movf    ILADDR,W
        subwf   FRAMEL,W   ; FRAMEL-ILADDR
        andlw   0x1f
        btfss   STATUS,Z
        goto    TRCMD22
        ; MTA (PIO)
        clrf    ILSTAT
        bsf     ILSTAT,FTLK
        goto    TRCMD29
TRCMD22 addlw   -1
        btfss   STATUS,Z
        goto    TRCMD29
        ; MTA (SER)
        clrf    ILSTAT
        bsf     ILSTAT,FTLK
        bsf     ILSTAT,FSER
TRCMD29 goto    TR11

TRCMD4  addlw   0xc0
        btfss   STATUS,Z
        goto    TR11
        ; cmd group 4
        movf    FRAMEL,W
        sublw   0x90   ; IFC
        btfss   STATUS,Z
        goto    TRCMD40
        clrf    ILSTAT
        goto    TRCMD49
TRCMD40 movf    FRAMEL,W
        sublw   0x92   ; REN
        btfss   STATUS,Z
        goto    TRCMD41
        bsf     ILFLG,FREN
        goto    TRCMD49
TRCMD41 movf    FRAMEL,W
        sublw   0x93   ; NRE
        btfss   STATUS,Z
        goto    TRCMD42
        bcf     ILFLG,FREN
        goto    TRCMD49
TRCMD42 movf    FRAMEL,W
        sublw   0x9a   ; AAU
        btfss   STATUS,Z
        goto    TRCMD43
        movlw   DEFADDR
        movwf   ILADDR	    
        goto    TRCMD49
TRCMD43 movf    FRAMEL,W
        sublw   0x9b   ; LPD
        btfss   STATUS,Z
        goto    TRCMD49
        bsf     ILFLG,FLPD
TRCMD49 goto    TR11
        
        
       

; -------------------------------        
; serial byte received.
; -------------------------------        
inserial
        ; ! do not desable intr! il frames can arrive anytime
        ; pay attn that Z flag is preserved during intr only if it reflects W status
        bcf     PORTA,LEDOFF      ; LED on
        call    inser
        bsf     PORTA,LEDOFF   ; LED off
        return

inser   movf    RCREG,W
        movwf   ACCX
        btfss   RCSTA,OERR
        goto    inSER1
; in case of USART error, reset it
        bcf     RCSTA,CREN
        nop
        bsf     RCSTA,CREN 
        bsf     SERSP,SERERR
inSER1         
        btfsc   NBSER,6
        goto    inSER8    ; overflow, ignore byte and set SEROVR
        ; store byte in serial input buffer
        movf    NBSER,W
        incf    NBSER
        addwf   PTSER,W
        andlw   0x3f
        addlw   BUFSER                
        movwf   FSR
        movf    ACCX,W
        movwf   INDF
        ; update status
        bsf     SERSP,SERRX
        sublw   0x0a
        btfsc   STATUS,Z
        bsf     SERSP,SERRXA
        btfsc   SERFLG,FRTS
        call    hndRTS        ; manage RTS hardware handshake line (if enabled)
        goto    inSER9
inSER8  bsf     SERSP,SEROVR     ; overflow
inSER9  return

hndRTS
        ; manage RTS hardware handshake line
        movlw   TRISB
        movwf   FSR
        btfss   NBSER,5
        goto    hndRTS7
        btfss   NBSER,4
        goto    hndRTS7
        btfss   NBSER,3
        goto    hndRTS9
        bsf     INDF,RTSO          ; RTS false (high) if NBSER>=56
        goto    hndRTS9
hndRTS7 bcf     INDF,RTSO          ; RTS true (low) if NBSER<48
hndRTS9 return

    
getserb
        movf    NBSER,W
        btfss   STATUS,Z
        goto    getsb5
        clrf    ACCX
getsb1  ; wait for a character for 25 ms
        btfsc   PIR1,RCIF
        goto    getsb2
        call    Dly100us
        decfsz  ACCX
        goto    getsb1
        bsf     STATUS,Z
        goto    getsb9
getsb2  call    inser
getsb5  movf    PTSER,W
        addlw   BUFSER
        movwf   FSR
        incf    PTSER
        bcf     PTSER,6   ; modulo 64
        decf    NBSER
        btfss   STATUS,Z
        goto    getsb6
        clrf    SERSP         ; no more data, clear RX, RXA, OVR, ERR
        clrf    PTSER
getsb6  movf    INDF,W
        movwf   ACCX
        btfsc   SERFLG,FRTS
        call    hndRTS        ; manage RTS hardware handshake line (if enabled)
        movf    ACCX,W
        bcf     STATUS,Z
getsb9  return


; ---------------------------------	
; put a byte in BUFOUT for serial
putserial
        movwf   SBYTE
        btfsc   NBSERO,4
        goto    putser8    ; buffer full
        ; store byte in serial output buffer
        movf    NBSERO,W
        incf    NBSERO
        addwf   PTSERO,W
        andlw   0x0f
        addlw   BUFOUT
        movwf   FSR
        movf    SBYTE,W
        movwf   INDF
        goto    putser7
putser8 bsf     ILFLG,FPBYT     ; pending byte
putser7 bsf     SERFLG,FDOUT    ; byte to be send
        return

; send a byte from BUFOUT to serial
; warning: intr are enabled here.
; pay attn that Z flag is preserved during intr only if it reflects W status
sndserial
        ; wait for TXIF
        btfss   PIR1,TXIF
        goto    sndser9
        ; wait for CTSI if enabled by FCTS
        btfss   SERFLG,FCTS
        goto    sndser2
        btfsc   PORTB,CTSI
        goto    sndser9
sndser2 ; send byte
        movf    PTSERO,W
        addlw   BUFOUT
        movwf   FSR
        movf    INDF,W
        movwf   TXREG
        incf    PTSERO
        bcf     PTSERO,4
        btfsc   ILFLG,FPBYT ; pending byte?
        goto    sndser3
        decf    NBSERO
        movf    NBSERO,W    ; (needed)
        btfss   STATUS,Z
        goto    sndser9
        clrf    PTSERO
        bcf     SERFLG,FDOUT
        goto    sndser9
sndser3 ; manage pending byte
        movf    SBYTE,W      ; store byte in buffer
        movwf   INDF
        bcf     ILFLG,FPBYT     ; clear pending byte
        ; retransmit pending byte to HPIL
        bcf     PORTA,LEDOFF   ; LED on
        bcf     INTCON, GIE   ; disable intr   
        clrf    FRAMEH
        movf    SBYTE,W
        movwf   FRAMEL
        call    sendIL     ; send HPIL frame (intr are enabled at return)
        bsf     PORTA,LEDOFF   ; LED off     
sndser9 return
        

    

; ----------------------------------------------------        
; send a frame to IL. Returns to input frame detection as fast as possible
sendIL
        movf    CMCON,W
        bcf     PIR1, CMIF   ; clear comparator IT
; sndframe
; send a frame from FRAME
MASK0   equ  0
MASKM   equ  1
MASKP   equ  8
        movf    FRAMEH,W
        movwf   FRREGH
        movf    FRAMEL,W
        movwf   FRREGL

        btfsc   FRREGH,2
        goto    sendf1

; send '0' sync bit
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKP  ; OUT+
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKP  ; OUT+
        movwf   PORTB
        goto    sendf2

sendf1
; send '1' sync bit
        movlw   MASKP  ; OUT+
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKP  ; OUT+
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
sendf2  nop
        movlw   MASK0  ; 
        movwf   PORTB
        nop

        movlw   D'9'   ; loop for the next 9 bits
        movwf   CPTBIT
                      
sendn       
        rlf     FRREGL
        rlf     FRREGH
        btfsc   FRREGH,2
        goto    send1
        nop
; send '0' bit
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKP  ; OUT+
        movwf   PORTB
        goto    send2

send1
; send '1' bit
        movlw   MASKP  ; OUT+
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        nop
send2   nop
        movlw   MASK0  ; 
        movwf   PORTB

        decfsz  CPTBIT
        goto    sendn

; last bit, optimized for fix with the HP71B HPIL module rev 1A
        
        nop
        rlf     FRREGL
        rlf     FRREGH
        btfsc   FRREGH,2
        goto    send11
        nop
        
; send '0' bit
        movlw   MASKM  ; OUT-
        movwf   PORTB
        nop
        movlw   MASKP  ; OUT+
        goto    send21

send11
; send '1' bit
        movlw   MASKP  ; OUT+
        movwf   PORTB
        nop
        nop
        nop
        movlw   MASKM  ; OUT-
send21  movwf   PORTB
        movlw   MASK0  ; 
        btfsc   PIR1, CMIF   ; check comparator IT
        goto    startframe   ; start of frame received!
        nop
        movwf   PORTB
        bsf     INTCON, GIE ; enable intr
        return

startframe
        movwf   PORTB
        call    Intr1        ; immediately call intr (GIE set by RETFIE)
        bsf     ILFLG,FRAV   ; frame available
        return
        

                


; ---------------------------------	

decframe
; decode frame from samples in BUF (20-5f). Result in FRAME, set ILFLG,FRAV if frame found
        movlw   BUF
        movwf   FSR
        movlw   40     ; 64 registers (128 samples)
        movwf   CPTSAM
        clrf    CPTBIT
        clrf    DECFLG ; flags for decoding
        bsf     DECFLG,fIDLE
; DECFLG flags: 
fBIT    equ     0    ; fBIT must be bit 0
fIDLE   equ     1    ; idle state
fDET    equ     2    ; bit detected
; sample bit positions:
fINp    equ     6
fINm    equ     7
fINp2   equ     2
fINm2   equ     3

        decf    FSR  ; preset FSR
decfra1
        incf    FSR

; find bits
; positive pulse? 
        btfsc   INDF,fINp
        goto    fndtr1
; negative pulse?
        btfss   INDF,fINm
        goto    fndtr2
; not in IDLE state ?        
        btfss   DECFLG,fIDLE
        goto    fndtr3

fndtr0  ; same for the 2 other bits of current sample
        btfsc   INDF,fINp2
        goto    fndtr1b
        btfss   INDF,fINm2
        goto    fndtr2b
        btfss   DECFLG,fIDLE
        goto    fndtr3b
        decfsz  CPTSAM
        goto    decfra1
        goto    fndtr6
        
fndtr1  clrf    DECFLG         ; not idle state
;        bcf     DECFLG,fBIT    ; last positive pulse will mean '0' bit
        goto    fndtr0

fndtr2  clrf    DECFLG         ; not idle state
        bsf     DECFLG,fBIT    ; last negative pulse will mean '1' bit
        goto    fndtr0
        
fndtr3                                
; check IDLE state after at least one pulse is found
; idle state found
        btfss   DECFLG,fDET
        goto    fndtr5
fndtr4
; idle found 2 times: one bit found, store it
        rrf     DECFLG,W
        rlf     FRREGL
        rlf     FRREGH
        incf    CPTBIT
        bsf     DECFLG,fIDLE
fndtr5  bsf     DECFLG,fDET
        goto    fndtr0

fndtr1b clrf    DECFLG         ; not idle state
        decfsz  CPTSAM
        goto    decfra1
        goto    fndtr6
fndtr2b clrf    DECFLG         ; not idle state
        bsf     DECFLG,fBIT 
        decfsz  CPTSAM
        goto    decfra1
        goto    fndtr6
fndtr3b btfss   DECFLG,fDET
        goto    fndtr5b
fndtr4b rrf     DECFLG,W
        rlf     FRREGL
        rlf     FRREGH
        incf    CPTBIT
        bsf     DECFLG,fIDLE
fndtr5b bsf     DECFLG,fDET
        decfsz  CPTSAM
        goto    decfra1
        
fndtr6
; end of samples. did we get exactly 11 bits?
        movf    CPTBIT,W
        sublw   D'11'
  ifdef FIX1A        
        btfsc   STATUS,Z
        goto    fndtr8              ; yes
        ; fix for HP71B HPIL module rev 1A
        ; check if start of frame was detected during sending
        btfss   ILFLG,FRAV
        goto    fndtr9  ; no
; yes. 10 bits received?        
        movf    CPTBIT,W
        sublw   D'10'
        btfss   STATUS,Z
        goto    fndtr9              ; no
; check if last sent frame was a data        
        btfsc   FRAMEH,2
        goto    fndtr9
        ; ok
        bcf     FRREGH,2            ; set 1st bit to 0
  else
        btfss   STATUS,Z
        goto    fndtr9              ; no
  endif        
        ;
fndtr8  bsf     ILFLG,FRAV          ; frame found
        movf    FRREGH,W
        andlw   7
        movwf   FRAMEH
        movf    FRREGL,W
        movwf   FRAMEL
fndtr9  return
       
 
blinkLED
; make blink the LED
        movlw   D'250'
        call    LEDon
        movlw   D'250'
        call    Delay
        
        movlw   D'250'
        call    LEDon
        movlw   D'250'
        call    Delay

        movlw   D'250'
        call    LEDon

        return   

LEDon
        movwf   CPTB
LEDon1
        clrwdt
        bcf     PORTA,LEDOFF   ; LED on
        call    Dly300us
        bsf     PORTA,LEDOFF   ; LED off
        call    Dly700us
        decfsz  CPTB,F
        goto    LEDon1
        return



; ------------------------------------------------------------------------        
        
; delay of W ms
Delay
        movwf   CPTB
d100ms  call    Dly1ms
        clrwdt
        decfsz  CPTB,F
        goto    d100ms
        return

; delay of 1 ms
Dly1ms
        call    Dly100us
        call    Dly100us
Dly800us      
        call    Dly100us
Dly700us      
        call    Dly100us
        call    Dly100us
        call    Dly100us
        call    Dly100us
Dly300us      
        call    Dly100us
        call    Dly100us
        call    Dly100us
        return

Dly100us                    ; delay 100us = 125 loops of 0.8us
        movlw   D'131'      ; (from 131 to 256)
d1ms    addlw   1           ; 1 cycle (0.2 us at 20MHz)
        btfss   STATUS,Z    ; 1
        goto    d1ms        ; 2
        return


        end


